package merge

import (
	"container/list"
	"fmt"
	"time"
)

type FanIn struct {
	lst     *list.List
	ticker  time.Ticker
	sync    func(l *list.List) error
	cancel  chan struct{}
	in      chan any
	maxSize int
}

func NewFanIn(d time.Duration, maxSize int, sync func(l *list.List) error) *FanIn {
	return &FanIn{
		lst:    list.New(),
		ticker: *time.NewTicker(d),
		sync: func(l *list.List) error {
			defer func() {
				if err := recover(); err != nil {
					l.Init()
					fmt.Println(fmt.Sprintf("panic %+v", err))
				}
			}()
			if err := sync(l); err != nil {
				l.Init()
				return err
			}
			l.Init()
			return nil
		},
		cancel:  make(chan struct{}),
		in:      make(chan any),
		maxSize: maxSize,
	}
}

func (f *FanIn) Start() {
	st := make(chan struct{})
	go f.start(st)
	<-st
}

func (f *FanIn) start(st chan struct{}) {
	st <- struct{}{}
	for {
		select {
		case <-f.cancel:
			_ = f.sync(f.lst)
			return
		case <-f.ticker.C:
			_ = f.sync(f.lst)
		case v := <-f.in:
			f.lst.PushBack(v)
			if f.lst.Len() >= f.maxSize {
				_ = f.sync(f.lst)
			}
		}
	}
}

func (f *FanIn) Stop() {
	f.cancel <- struct{}{}
}

func (f *FanIn) Add(v any) {
	f.in <- v
}
